@@ -7,6 +7,7 @@ import sys
 import tempfile
 
 from plypatch import exc
+from plypatch import fixup_patch
 from plypatch import git
 from plypatch import utils
 from plypatch import version
@@ -17,86 +18,9 @@ __version__ = version.__version__
 RE_PATCH_IDENTIFIER = re.compile('Ply-Patch: (.*)')
 
 
-FROM_SHA1_VALUE = 'ply'
-PATCH_GIT_VERSION = '1.8.3'
-
-
-def _replace_from_sha1(lines):
-    """The SHA1 on the 'From' line of the patch will differ each time the
-    patch file is regenerated. To keep this from causing chatty diffs,
-    replace the SHA1 with a hardcoded value.
-    """
-    for line_idx, line in enumerate(lines):
-        if line.startswith('From'):
-            break
-    else:
-        raise Exception("Malformed patch: 'From' not found")
-
-    parts = lines[line_idx].split(' ')
-    parts[1] = FROM_SHA1_VALUE
-
-    lines[line_idx] = ' '.join(parts)
-
-
-def _replace_git_version(lines):
-    """The version of git is embedded in the patch which will differ causing
-    chatty-diffs and unecessary conflicts.
-
-    It's terribly evil, but hardcoding this is the easiest way to avoid these
-    conflicts.
-    """
-    for reverse_line_idx, line in enumerate(reversed(lines)):
-        if line[0].isdigit() and '.' in line:
-            break
-    else:
-        raise Exception("Malformed patch: Git version not found")
-
-    # The last line counting backwards (0) becomes -(0) - 1 which becomes -1
-    # counting foward
-    line_idx = -reverse_line_idx - 1
-    lines[line_idx] = '%s\n' % PATCH_GIT_VERSION
-
-
-def _remove_ply_patch_annotation(lines):
-    match_idxs = []
-    for idx, line in enumerate(lines):
-        if 'Ply-Patch:' in line:
-            match_idxs.append(idx)
-
-    for idx in reversed(match_idxs):
-        del lines[idx]
-
-
-def _remove_trailing_extra_blank_lines_from_subject(lines):
-    """Different versions of git will output different amounts of trailing
-    whitespace at the end of a subject headers, so we normalize it by only
-    allowing a single trailing blank line.
-    """
-    for idx, line in enumerate(lines):
-        if line.startswith('diff --git'):
-            break
-    else:
-        return
-
-    if idx < 2:
-        return
-
-    # If we have two blanks above first `diff --git` remove one of them
-    if not lines[idx - 1].strip() and not lines[idx - 2].strip():
-        del lines[idx - 1]
-
-
-def _fixup_patch(from_file, to_file):
-    lines = from_file.readlines()
-    _replace_from_sha1(lines)
-    _replace_git_version(lines)
-    _remove_ply_patch_annotation(lines)
-    _remove_trailing_extra_blank_lines_from_subject(lines)
-    to_file.write(''.join(lines))
-
-
-
 class Repo(git.Repo):
+    NON_INTERACTIVE = False
+
     def _add_annotation(self, prefix, value):
         """Add annotation to the last commit."""
         commit_msg = self.log(count=1, pretty='%B')
@@ -218,10 +142,11 @@ class WorkingRepo(Repo):
             if not self.patch_repo_path:
                 raise exc.NoLinkedPatchRepo
 
-            self._patch_repo =  PatchRepo(
-                             self.patch_repo_path,
-                             quiet=self.quiet,
-                             supress_warnings=self.supress_warnings)
+            self._patch_repo = PatchRepo(
+                self.patch_repo_path,
+                quiet=self.quiet,
+                supress_warnings=self.supress_warnings)
+
         return self._patch_repo
 
     @property
@@ -315,7 +240,8 @@ class WorkingRepo(Repo):
         if len(source_paths) > 1:
             raise Exception("Too many patches generated")
 
-        self.patch_repo.sync_patches(source_paths, parent_patch_name)
+        self.patch_repo.sync_patches(source_paths, parent_patch_name,
+                                     last_patch_name=patch_name)
 
         self._add_patch_annotation(patch_name)
         self.restore(fetch_remotes=False)  # Apply remaining patches
@@ -369,7 +295,7 @@ class WorkingRepo(Repo):
             raise exc.GitConfigRequired('user.name')
 
     def restore(self, three_way_merge=True, commit_msg=None,
-                fetch_remotes=True):
+                fetch_remotes=True, customize_commit_msg=False):
         """Applies a series of patches to the working repo's current
         branch.
         """
@@ -450,15 +376,41 @@ class WorkingRepo(Repo):
         if os.path.exists(self._restore_stats_path):
             os.unlink(self._restore_stats_path)
 
-        if not commit_msg:
-            commit_msg = 'Refreshing patches: %d updated, %d removed' % (
-                updated, removed)
-
         # Commit to patch repo
         if not self.patch_repo.uncommitted_changes():
             return
         based_on = self._last_upstream_commit_hash()
-        self.patch_repo.commit(msgs=[commit_msg])
+
+        template = None
+
+        # Determine whether we need to prompt the user to edit commit message
+        if commit_msg:
+            if customize_commit_msg:
+                msgs = []
+
+                # Use template so user can edit
+                with tempfile.NamedTemporaryFile(delete=False) as templ:
+                    templ.write(commit_msg)
+
+                template = templ.name
+            else:
+                # Use commit message as-is
+                msgs = [commit_msg]
+        else:
+            if customize_commit_msg:
+                # Require user edit commit message
+                msgs = []
+            else:
+                # Use this default commit message without editing
+                msgs = ['Refreshing patches: %d updated, %d removed' % (
+                        updated, removed)]
+
+        try:
+            self.patch_repo.commit(msgs=msgs, template=template)
+        finally:
+            if template:
+                os.unlink(template)
+
         self.patch_repo._add_annotation('Ply-Based-On', based_on)
 
     def rollback(self, lose_uncommitted=False):
@@ -495,13 +447,14 @@ class WorkingRepo(Repo):
         filenames = self.format_patch(
             since, keep_subject=True, no_numbered=True, no_stat=True)
 
-        # Rewrite first-line 'From <commit-hash>' -> 'From ply'
         source_paths = []
         for filename in filenames:
             from_path = os.path.join(self.path, filename)
             with tempfile.NamedTemporaryFile(delete=False) as to_file:
                 with open(from_path) as from_file:
-                    _fixup_patch(from_file, to_file)
+                    original = from_file.read()
+                    fixed = fixup_patch.fixup_patch(original)
+                    to_file.write(fixed)
 
             # Strip 0001- prefix that git-format-patch uses
             source_path = os.path.join(self.path, filename.split('-', 1)[1])
@@ -545,7 +498,8 @@ class WorkingRepo(Repo):
         # We have to commit to the patch-repo AFTER rolling-back and
         # reapplying so that we have the patch-annotations necessary to figure
         # out the correct Ply-Based-On annotation in the patch-repo.
-        self.restore(commit_msg=commit_msg, fetch_remotes=False)
+        self.restore(commit_msg=commit_msg, fetch_remotes=False,
+                     customize_commit_msg=not self.NON_INTERACTIVE)
 
     @property
     def status(self):
@@ -617,7 +571,8 @@ class PatchRepo(Repo):
 
         self.add('series')
 
-    def _determine_what_changed(self, source_paths, parent_patch_name):
+    def _determine_what_changed(self, source_paths, parent_patch_name,
+                                last_patch_name=None):
         added = set()
         updated = set()
         skipped = set()
@@ -636,20 +591,31 @@ class PatchRepo(Repo):
             else:
                 added.add(patch_name)
 
-        # Skip all patches up to parent_patch_name
-        if parent_patch_name:
-            patch_names = self._non_recursive_series(self.series_path)
-            for patch_name in patch_names:
-                skipped.add(patch_name)
+        skip_before = True
+        skip_after = False
+        patch_names = self._non_recursive_series(self.series_path)
+        for patch_name in patch_names:
+            # Skip all patches UP TO AND INCLUDING parent_patch_name
+            if parent_patch_name:
+                if skip_before:
+                    skipped.add(patch_name)
                 if patch_name == parent_patch_name:
-                    break
+                    skip_before = False
+
+            # Skip all patches AFTER BUT NOT INCLUDING last_patch_name
+            if last_patch_name:
+                if skip_after:
+                    skipped.add(patch_name)
+                if patch_name == last_patch_name:
+                    skip_after = True
 
         series = set(self.series)
         removed = series - added - updated - skipped
 
         return added, updated, skipped, removed
 
-    def sync_patches(self, source_paths, parent_patch_name):
+    def sync_patches(self, source_paths, parent_patch_name,
+                     last_patch_name=None):
         """Sync patches into working repo, adding, updating, and removing
         patches as necessary.
 
@@ -659,11 +625,15 @@ class PatchRepo(Repo):
         `parent_patch_name` represents where in the `series` file we should
         insert the new patch set.
 
+        `last_patch_name` represents the last patch we've seen. This is used
+        in during a resolve to so that we don't delete patches that we haven't
+        seen yet.
+
         `None` indicates that the patch-set doesn't have a parent so it should
         be inserted at the beginning of the series file.
         """
         added, updated, skipped, removed = self._determine_what_changed(
-                source_paths, parent_patch_name)
+            source_paths, parent_patch_name, last_patch_name=last_patch_name)
 
         source_lookup = {}
         dest_lookup = {}
