@@ -159,11 +159,15 @@ class ResolveCommand(CLICommand):
 class RestoreCommand(CLICommand):
     __command__ = 'restore'
 
+    def add_arguments(self, subparser):
+        subparser.add_argument('-m', '--message', action='store_true',
+                               help='Prompt for a custom commit message')
+
     def do(self, args):
         """Apply the patch series to the the current branch of the
         working-repo"""
         try:
-            self.working_repo.restore()
+            self.working_repo.restore(customize_commit_msg=args.message)
         except plypatch.exc.GitConfigRequired as e:
             die("Required git config '%s' is unset." % e)
         except plypatch.exc.RestoreInProgress:
